@@ -1,5 +1,5 @@
-/* wc - print the number of bytes, words, and lines in files
-   Copyright (C) 85, 91, 1995-2004 Free Software Foundation, Inc.
+/* wc - print the number of lines, words, and bytes in files
+   Copyright (C) 85, 91, 1995-2005 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 /* Written by Paul Rubin, phr@ocf.berkeley.edu
    and David MacKenzie, djm@gnu.ai.mit.edu. */
@@ -50,11 +50,6 @@
 #include "inttostr.h"
 #include "safe-read.h"
 
-/* Some systems, like BeOS, have multibyte encodings but lack mbstate_t.  */
-#if HAVE_MBRTOWC && defined mbstate_t
-# define mbrtowc(pwc, s, n, ps) (mbrtowc) (pwc, s, n, 0)
-#endif
-
 #ifndef HAVE_DECL_WCWIDTH
 "this configure-time declaration test was not run"
 #endif
@@ -226,9 +221,6 @@ wc (int fd, char const *file_x, struct fstatus *fstatus)
     }
   count_complicated = print_words | print_linelength;
 
-  /* We need binary input, since `wc' relies on `lseek' and byte counts.  */
-  SET_BINARY (fd);
-
   /* When counting only bytes, save some line- and word-counting
      overhead.  If FD is a `regular' Unix file, using lseek is enough
      to get its `size' in bytes.  Otherwise, read blocks of BUFFER_SIZE
@@ -274,7 +266,7 @@ wc (int fd, char const *file_x, struct fstatus *fstatus)
 	 but not chars or words.  */
       while ((bytes_read = safe_read (fd, buf, BUFFER_SIZE)) > 0)
 	{
-	  register char *p = buf;
+	  char *p = buf;
 
 	  if (bytes_read == SAFE_READ_ERROR)
 	    {
@@ -504,14 +496,16 @@ wc (int fd, char const *file_x, struct fstatus *fstatus)
 static bool
 wc_file (char const *file, struct fstatus *fstatus)
 {
-  if (STREQ (file, "-"))
+  if (! file || STREQ (file, "-"))
     {
       have_read_stdin = true;
+      if (O_BINARY && ! isatty (STDIN_FILENO))
+	freopen (NULL, "rb", stdin);
       return wc (STDIN_FILENO, file, fstatus);
     }
   else
     {
-      int fd = open (file, O_RDONLY);
+      int fd = open (file, O_RDONLY | O_BINARY);
       if (fd == -1)
 	{
 	  error (0, errno, "%s", file);
@@ -595,6 +589,7 @@ compute_number_width (int nfiles, struct fstatus const *fstatus)
 int
 main (int argc, char **argv)
 {
+  int i;
   bool ok;
   int optc;
   int nfiles;
@@ -653,23 +648,13 @@ main (int argc, char **argv)
   fstatus = get_input_fstatus (nfiles, argv + optind);
   number_width = compute_number_width (nfiles, fstatus);
 
-  if (! argv[optind])
-    {
-      have_read_stdin = true;
-      ok = wc (STDIN_FILENO, NULL, &fstatus[0]);
-    }
-  else
-    {
-      int i;
+  ok = true;
+  for (i = 0; i < nfiles; i++)
+    ok &= wc_file (argv[optind + i], &fstatus[i]);
 
-      ok = true;
-      for (i = 0; i < nfiles; i++)
-	ok &= wc_file (argv[optind + i], &fstatus[i]);
-
-      if (nfiles > 1)
-	write_counts (total_lines, total_words, total_chars, total_bytes,
-		      max_line_length, _("total"));
-    }
+  if (1 < nfiles)
+    write_counts (total_lines, total_words, total_chars, total_bytes,
+		  max_line_length, _("total"));
 
   free (fstatus);
 
